-- Add1toCOBOL.Lua is a library for Soupault plugins.
-- It Allows the use of add1tocobol namespaced functions and variables
-- in any plugin all at once just by one dofile().
--
-- See: README.md
-- 
-- Usage:
-- Plugin.require_version('1.8')
-- dofile('plugins/add1tocobol/add1tocobol.lua')
--
-- Author: Aoirthoir
-- License: MIT
-- Add1toCOBOL.Lua: http://soupault.add1tocobol.com/add1tocobol-lua
-- Soupault: https://soupault.neocities.org/
--
-- *********************************************************************
-- * This library creates the following Widget Settings:
-- *********************************************************************
-- * action = 
-- * elements_apply =
-- * images =
-- * plugins = 
-- * raw = 
-- * selectors =
-- * selectors_apply =
-- * templates =
-- *********************************************************************
-- * This library has the following Plugins:
-- *********************************************************************
-- * add-class.lua
-- * add-style.lua
-- * block-maker.lua
-- * colorize-backgrounds.lua
-- * colorize-characters.lua
-- * colorize-elements.lua
-- * colorize-words.lua
-- * content-to-class.lua
-- * delete-elements.lua
-- * do-code.lua
-- * include-tag.lua
-- * insert-html.lua
-- * insert-html-if.lua
-- * pre-me.lua
-- * tree-image-file.lua
-- * tree-image-src.lua
-- * tree-template-include.lua

-- *********************************************************************
-- * This library contains the following Functions for use by Plugins:
-- *********************************************************************
-- * ADD1.add_attribute()
-- * ADD1.apply_random_color()
-- * ADD1.create_array()
-- * ADD1.default_selectors()
-- * ADD1.find_file_in_tree()
-- * ADD1.get_config_or_default()
-- * ADD1.get_config_array_or_default()
-- * ADD1.get_file()
-- * ADD1.get_random_color()
-- * ADD1.get_site_directories()
-- * ADD1.loop_selectors()
-- * ADD1.loop_elements()
-- * ADD1.message()
-- * msg() - quick access to ADD1.message()
-- * ADD1.process_element()
-- * ADD1.set_standard_variables() - called right before file's end.
-- * ADD1.trim_left()
-- * ADD1.wrap_characters_with_tag()
-- * ADD1.wrap_words_with_tag()
-- *********************************************************************
-- * This library creates the following Variables for use by Plugins:
-- *********************************************************************
-- * ADD1.action
-- * ADD1.builddir
-- * ADD1.elementsApply
-- * ADD1.imagesdir
-- * ADD1.pluginsdir
-- * ADD1.raw
-- * ADD1.selectors 
-- * ADD1.selectorsApply
-- * ADD1.selectorsCount
-- * ADD1.selectorsIndex
-- * ADD1.sitedir
-- * ADD1.templatesdir
-- * head
-- * body
-- *********************************************************************

-- *********************************************************************
-- * Set the Namespace to ADD1
-- *********************************************************************
ADD1 = {}
-- *********************************************************************
-- * ADD1.add_attribute()
-- *********************************************************************
  function ADD1.add_attribute(element,attributeName,attributeValue)
    local attribute = HTML.get_attribute(element,attributeName)
    if attribute then
      attribute = attribute..attributeValue
    else
      attribute = attributeValue
    end  
    HTML.set_attribute(element,attributeName,attribute)
  end

-- *********************************************************************
-- * ADD1.apply_random_color()
-- *********************************************************************
  function ADD1.apply_random_color(element,cssProperty,rejectedColor)
    local results
    css = cssProperty..':'..ADD1.get_random_color(rejectedColor)..';'
    style = HTML.get_attribute(element,'style')
    if style then
      style = style..css
    else
      style = css
    end
    HTML.set_attribute(element,'style',style)    
    return 'ok'
  end

-- *********************************************************************
-- * ADD1.create_array()
-- *********************************************************************
  function ADD1.create_array(value,separator)
    local results = {}
    local resultsCount
    if not separator then separator = ',' end
    if not value then
      results[1] = 'ADD1.create_array(): No variable sent'
    else
      results = Regex.split(value,separator)
    end
    resultsCount = size(results)
    return results, resultsCount, 1
  end

-- *********************************************************************
-- * ADD1.default_selectors()
-- *********************************************************************
  function ADD1.default_selectors(default)
    if not default then default = '*' end
    if not config.selectors then
      ADD1.selectors, 
      ADD1.selectorsCount,
      ADD1.selectorsIndex
        = ADD1.create_array(default)
    end
  end

-- *********************************************************************
-- * ADD1.find_file_in_tree()
-- *********************************************************************
  function ADD1.find_file_in_tree(parentlist,file)
    local results = 'NO FILE FOUND'

    local parents = {}
    local parentsCount 
    local parentsIndex
    local parent    

    local directories = {}
    local directoriesCount
    local directoriesIndex
    local directory

    local filepaths = {}
    local filepathsCount
    local filepathsIndex
    local filepath

    local currentpath
    local filefound

    parents = Regex.split(parentlist, ',')
    parentsCount = size(parents)
    parentsIndex = 1
    while (parentsIndex <= parentsCount) do
      parent = parents[parentsIndex]
      directories = Regex.split(target_dir, '/')
      directories[1] = parent
      directoriesCount = size(directories)
      currentpath = ''
      directoriesIndex = 1
      while (directoriesIndex <= directoriesCount) do
        directory = directories[directoriesIndex]
        currentpath = currentpath..directory..'/'
        filepaths[directoriesIndex] = currentpath..file
        directoriesIndex = directoriesIndex + 1
      end
      filepathsCount = size(filepaths)
      filepathsIndex = filepathsCount
      while (filepathsIndex > 0) do
        filepath = filepaths[filepathsIndex]
        filefound = readfrom(filepath)
        if filefound then
          filepathsIndex = 0
          results = filepath
        end
        filepathsIndex = filepathsIndex - 1
      end
      if results == 'NO FILE FOUND' then
        parentsIndex = parentsIndex + 1
      else
        parentsIndex = parentsCount + 1
      end        
    end
    return results
  end

-- *********************************************************************
-- * ADD1.get_config_or_default()
-- *********************************************************************
  function ADD1.get_config_or_default(variable,default)
    local results
    if not variable then
      results = 'ADD1.get_config_or_default(): No config variable sent'
    else
      if not default then default = variable end
      if not config[variable] then
        results = default
      else
        results = config[variable]
      end
    end
    return results
  end

-- *********************************************************************
-- * ADD1.get_config_array_or_default()
-- *********************************************************************
  function ADD1.get_config_array_or_default(variable,default,separator)
    local results = {}
    local resultsCount
    if not separator then separator = ',' end
    if not variable then
      results[1] = 'ADD1.get_config_array_or_default(): No variable sent'
    else
      if not default then 
        default = 'ADD1.get_config_array_or_default(): No default sent'
      end
      if not config[variable] then
        results[1] = default
      else
        results = Regex.split(config[variable],separator)
      end
    end
    resultsCount = size(results)
    return results, resultsCount, 1
  end

-- *********************************************************************
-- * ADD1.get_file()
-- *********************************************************************
  function ADD1.get_file(filepath)
    local results
    local isPath = strlen(filepath)
    if isPath == 0 then
      results = 'NOT A FILENAME'
    else
      filefound = readfrom(filepath)
      if not filefound then
        results = 'FILE NOT FOUND'
      else
        data = Sys.read_file(filepath)
        if not data then
          results = 'NOT DATA IN FILE'
        else
          results = 'SUCCESS'
        end
      end
    end
    return results, data
  end

-- *********************************************************************
-- * ADD1.get_random_color()
-- *********************************************************************
  function ADD1.get_random_color(rejectedColor)
    local red
    local green
    local blue
    local keepgoing = 1
    while keepgoing == 1 do
      red   = Sys.random(255)
      green = Sys.random(255)
      blue  = Sys.random(255)
      if rejectedColor == red..','..green..','..blue then
        keepgoing = 1
      else
        keepgoing = 0
      end
    end
    return 'rgb('..red..', '..green..', '..blue..')'
  end

-- *********************************************************************
-- * ADD1.get_site_directories()
-- *********************************************************************
  function ADD1.get_site_directories()
    local paths        = {}
    local sitedir      = ''
    local builddir     = ''
    local templatesdir = ''
    local pluginsdir   = ''
    local imagesdir    = ''
    paths        = Regex.split(page_file, '/')
      sitedir    = paths[1]
    paths        = Regex.split(target_dir, '/')
      builddir   = paths[1]
    templatesdir = ADD1.get_config_or_default('templates')
    pluginsdir   = ADD1.get_config_or_default('plugins')
    imagesdir    = sitedir..'/'..ADD1.get_config_or_default('images')
    return sitedir, builddir, templatesdir, pluginsdir, imagesdir
  end

-- *********************************************************************
-- * ADD1.loop_selectors()
-- *********************************************************************
  function ADD1.loop_selectors(code,selectors,selectorsApply,elementsApply)
    local results  = 'none'
    if not selectors then selectors = ADD1.selectors end
    if (selectorsApply ~= 'first' and selectorsApply ~= 'all') then
      selectorsApply = ADD1.selectorsApply
    end
    local selectorsCount = size(selectors)
    local selectorsIndex = 1    
    local selector
    while (selectorsIndex <= selectorsCount) do
      selector = selectors[selectorsIndex]
      results = ADD1.loop_elements(page,selector,elementsApply,code)
      if (selectorsApply == 'first' and results == 'success') then
        selectorsIndex = selectorsCount + 1
      elseif selectorsApply == 'all' then
        selectorsIndex = selectorsIndex + 1
      else
        selectorsIndex = selectorsIndex + 1
      end
    end
    return results
  end
  
-- *********************************************************************
-- * ADD1.loop_elements()
-- *********************************************************************
  function ADD1.loop_elements(parent,selector,elementsApply,code)
    local results = 'none'
    if (elementsApply ~= 'first' and elementsApply ~= 'all') then
      elementsApply = ADD1.elementsApply
    end
    local elements
    local elementsCount
    local elementsIndex
    elements = HTML.select(parent, selector)
    elementsCount = size(elements)
    elementsIndex = 1
    while (elementsIndex <= elementsCount) do
      element = elements[elementsIndex]
      dostring(code)
      results = 'success'
      if elementsApply == 'first' then
        elementsIndex = elementsCount + 1
      elseif elementsApply == 'all' then
        elementsIndex = elementsIndex + 1
      else
        elementsIndex = elementsIndex + 1
      end
    end
    return results
  end

-- *********************************************************************
-- * ADD1.message()
-- *********************************************************************
  function ADD1.message(plugin,message)
    local results
    results = 'page_file:'..page_file..
              ' plugin:'..plugin..
              ' message:'..message
    return results
  end

-- *********************************************************************
-- * ADD1.process_element()
-- *********************************************************************
  function ADD1.process_element(action, element, data, raw)
    local results = 'Success'
    if not element then
      results = 'ADD1.process_element(): No element sent'
    else
      if not action then
        action = 'append_child'
      end
      if raw == 'raw' then
        html = HTML.create_text(data)
      else
        html = HTML.parse(data)
      end
      if     action == 'append_child'    then
        HTML.append_child(element, html)
      elseif action == 'prepend_child'   then
        HTML.prepend_child(element, html)
      elseif action == 'insert_before'   then
        HTML.insert_before(element, html)
      elseif action == 'insert_after'    then
        HTML.insert_after(element, html)
      elseif action == 'replace_element' then
        HTML.replace_element(element, html)
      elseif action == 'replace_content' then
        HTML.replace_content(element, html)
      else
        results = 'ADD1.process_element(): Action:'..action..' wrong'
      end
    end
    return results
  end
  
-- *********************************************************************
-- * ADD1.set_standard_variables()
-- *********************************************************************
  function ADD1.set_standard_variables()
    ADD1.sitedir,
    ADD1.builddir,
    ADD1.templatesdir,
    ADD1.pluginsdir,
    ADD1.imagesdir
      = ADD1.get_site_directories()

    ADD1.selectors, 
    ADD1.selectorsCount,
    ADD1.selectorsIndex
      = ADD1.get_config_array_or_default('selectors','')

    ADD1.selectorsApply
      = ADD1.get_config_or_default('selectors_apply','all')
    ADD1.elementsApply
      = ADD1.get_config_or_default('elements_apply','all')    

    ADD1.action
      = ADD1.get_config_or_default('action','append_child')
    ADD1.raw
      = ADD1.get_config_or_default('raw','')
      
    body =  HTML.select_one(page, 'body') 
    head =  HTML.select_one(page, 'head')
  end
  
-- *********************************************************************
-- * ADD1.trim_left()
-- *********************************************************************
  function ADD1.trim_left(value,trim)
    local results
    local length
    if not value then
      results = 'ADD1.trim_left(): NO VARIABLE PASSED'
    elseif not trim then
      results = value
    else
      length = strlen(trim) + 1
      results = strsub(value,length)
    end
    return results
  end
  
-- *********************************************************************
-- * ADD1.wrap_characters_with_tag()
-- *********************************************************************
  function ADD1.wrap_characters_with_tag(element,tag)
      local results = ''
      local character = ''
      contents, 
      contentsCount, 
      contentsIndex =
        ADD1.create_array(HTML.strip_tags(element),'')      
      if not tag then tag = 'span' end
      while (contentsIndex <= contentsCount) do
        character = contents[contentsIndex]
        if character == ' ' then
          results = results..character
        else
          results = results..'<'..tag..'>'..character..'</'..tag..'>'
        end
        contentsIndex = contentsIndex + 1
      end
      return results
  end

-- *********************************************************************
-- * ADD1.wrap_words_with_tag()
-- *********************************************************************
  function ADD1.wrap_words_with_tag(element,tag)
      local results = ''
      local word = ''
      contents, 
      contentsCount, 
      contentsIndex =
        ADD1.create_array(HTML.strip_tags(element),' ')
      if not tag then tag = 'span' end
      while (contentsIndex <= contentsCount) do
        word = contents[contentsIndex]
        results = results..'<'..tag..'>'..word..'</'..tag..'> '
        contentsIndex = contentsIndex + 1
      end
      return results
  end
  
-- *********************************************************************
-- * Call the function to set all the standard variables.
-- *********************************************************************
  ADD1.set_standard_variables()
  msg = ADD1.message

-- *********************************************************************
-- * RETURN ADD1
-- *********************************************************************  
  return ADD1

